@@ -1,25 +1,24 @@
 #!/usr/bin/python3
 
 #
-#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
-#   All rights reserved. Copyright (c) 2024 VECTIONEER.
+#   Developer: Alexey Zakharov (alexey.zakharov@vectioneer.com)
+#   All rights reserved. Copyright (c) 2024 - 2026 VECTIONEER.
 #
 
 from .robot_command import *
 
-"""Checks if all the values in the list are equal to the objective value.
-
-    Args:
-        list_of_values(list(any)): list of the values to compare
-        objective_value(any): an objective value
-
-    Returns:
-        bool: True if operation all is equal, False if not
-
-"""
-
 
 def isEqual(list_of_values, objective_value):
+    """Checks if all the values in the list are equal to the objective value.
+
+        Args:
+            list_of_values(list(any)): list of the values to compare
+            objective_value(any): an objective value
+
+        Returns:
+            bool: True if all values are equal to objective, False otherwise
+
+    """
     return all(objective_value == value for value in list_of_values)
 
 
@@ -29,7 +28,7 @@ class MultiRobotCommand(object):
         Args:
             req(motorcortex.Request): reference to a Request instance
             motorcortex_types(motorcortex.MessageTypes): reference to a MessageTypes instance
-            systems(list(int)): a list of systems id
+            systems_id(list(int)): a list of systems id
 
     """
 
@@ -53,14 +52,22 @@ class MultiRobotCommand(object):
         return f"{name}{system:02d}"
 
     def play(self, systems_id=None, wait_time=1.0):
-        """Plays the program.
+        """Plays the program from the current trajectory position for multiple robots.
+
+        If any robot has deviated from the trajectory, a guard violation will occur
+        and that robot must return to the trajectory start position using moveToStart()
+        before motion can continue.
 
             Args:
-                wait_time(double): short delay after which actual state of the interpreter is requested
-                systems_id(list(int)): an optional list of the systems id, a subset of the object defined list
+                systems_id(list(int)): optional list of systems id, subset of object defined list
+                wait_time(float): short delay after which the actual state of the interpreter is requested
 
             Returns:
-                InterpreterStates: actual state of the program interpreter
+                list(InterpreterStates): actual states of the program interpreters
+
+            See Also:
+                moveToStart: Use to return robots to trajectory start after guard violation.
+                playFromActual: Use to continue from the current position without returning to trajectory.
 
         """
 
@@ -77,15 +84,57 @@ class MultiRobotCommand(object):
 
         return self.getState(systems_id)
 
-    def pause(self, systems_id=None, wait_time=1.0):
-        """Plays the program.
+    def playFromActual(self, systems_id=None, wait_time=1.0):
+        """Plays the program from the robots' current actual positions.
+
+        Unlike play(), this method allows robots to continue from their current positions
+        even if they have deviated from the original trajectory. The interpreter automatically
+        moves each robot back to the trajectory and resumes program execution without
+        requiring manual intervention (no button hold needed).
 
             Args:
-                wait_time(double): short delay after which actual state of the interpreter is requested
-                systems_id(list(int)): an optional list of the systems id, a subset of the object defined list
+                systems_id(list(int)): optional list of systems id, subset of object defined list
+                wait_time(float): short delay after which the actual state of the interpreter is requested
 
             Returns:
-                InterpreterStates: actual state of the program interpreter
+                list(InterpreterStates): actual states of the program interpreters
+
+            See Also:
+                play: Use when robots are on the trajectory or after manual moveToStart.
+                moveToStart: Manual alternative requiring button hold to return to trajectory.
+
+        """
+
+        if not systems_id:
+            systems_id = self.__systems_id
+
+        paramList = []
+        for system_id in systems_id:
+            paramList.append({"path": f'root/{self.__addSystem("MotionInterpreter", system_id)}/interpreterCommand',
+                              "value": InterpreterEvents.PLAY_PROGRAM_FROM_ACTUAL_E.value})
+
+        self.__req.setParameterList(paramList)
+        time.sleep(wait_time)
+
+        return self.getState(systems_id)
+
+    def pause(self, systems_id=None, wait_time=1.0):
+        """Pause the program for multiple robots.
+
+        Stops motion while preserving the current program position. Use play() or
+        playFromActual() to resume execution.
+
+            Args:
+                systems_id(list(int)): optional list of systems id, subset of object defined list
+                wait_time(float): short delay after which the actual state of the interpreter is requested
+
+            Returns:
+                list(InterpreterStates): actual states of the program interpreters
+
+            See Also:
+                play: Resume from the trajectory position (may require moveToStart if deviated).
+                playFromActual: Resume from the current actual position.
+                stop: Stop the program and return to the beginning.
 
         """
 
@@ -103,11 +152,13 @@ class MultiRobotCommand(object):
         return self.getState(systems_id)
 
     def getState(self, systems_id=None):
-        """
-            systems_id(list(int)): an optional list of the systems id, a subset of the object defined list
+        """Get the current interpreter states for multiple robots.
+
+            Args:
+                systems_id(list(int)): optional list of systems id, subset of object defined list
 
             Returns:
-                InterpreterStates: actual state of the interpreter
+                list(InterpreterStates): actual states of the interpreters
 
         """
 
@@ -130,20 +181,27 @@ class MultiRobotCommand(object):
         """Switch robot to Engage state.
 
             Returns:
-                bool: True if operation is completed, False if failed
+                bool: True if the operation is completed, False if failed
 
         """
         return self.__robot_command_list[self.__systems_id[0]].engage()
 
     def stop(self, systems_id=None, wait_time=1.0):
-        """Stop the program.
+        """Stop the program for multiple robots.
+
+        Stops motion and resets program execution to the beginning. Unlike pause(),
+        the program cannot be resumed from the current position.
 
             Args:
-                systems_id(list(int)): an optional list of the systems id, a subset of the object defined list
-                wait_time(double): short delay after which actual state of the interpreter is requested
+                systems_id(list(int)): optional list of systems id, subset of object defined list
+                wait_time(float): short delay after which the actual state of the interpreter is requested
 
             Returns:
-                InterpreterStates: actual state of the program interpreter
+                list(InterpreterStates): actual states of the program interpreters
+
+            See Also:
+                pause: Pause without resetting the program position.
+                reset: Stop and clear the interpreter buffer.
 
         """
 
@@ -161,14 +219,20 @@ class MultiRobotCommand(object):
         return self.getState(systems_id)
 
     def reset(self, systems_id=None, wait_time=1.0):
-        """Stop the program and clear the interpreter buffer.
+        """Stop the program and clear the interpreter buffer for multiple robots.
+
+        Performs a full reset of the interpreters, stopping any motion and clearing
+        all buffered commands. Use this to completely restart program execution.
 
             Args:
-                systems_id(list(int)): an optional list of the systems id, a subset of the object defined list
-                wait_time(double): short delay after which actual state of the interpreter is requested
+                systems_id(list(int)): optional list of systems id, subset of object defined list
+                wait_time(float): short delay after which the actual state of the interpreter is requested
 
             Returns:
-                InterpreterStates: actual state of the program interpreter
+                list(InterpreterStates): actual states of the program interpreters
+
+            See Also:
+                stop: Stop without clearing the buffer.
 
         """
 
@@ -186,14 +250,25 @@ class MultiRobotCommand(object):
         return self.getState(systems_id)
 
     def moveToStart(self, timeout_s, systems_id=None):
-        """Move arm to the start of the program.
+        """Move arms to the start of the trajectory for multiple robots.
+
+        Used to return robots to the trajectory start position after a guard violation
+        (when a robot has deviated from the planned trajectory). This is required before
+        play() can resume motion.
 
             Args:
-                timeout_s(double): timeout in seconds
-                systems_id(list(int)): an optional list of the systems id, a subset of the object defined list
+                timeout_s(float): timeout in seconds
+                systems_id(list(int)): optional list of systems id, subset of object defined list
 
             Returns:
-                bool: True if operation is completed, False if failed
+                list(InterpreterStates): actual states of the program interpreters
+
+            Raises:
+                Exception: if any robot fails to move to start position within timeout
+
+            See Also:
+                play: Resumes program execution after robots are at trajectory start.
+                playFromActual: Alternative that continues from current position without moveToStart.
 
         """
         if not systems_id:
@@ -212,13 +287,13 @@ class MultiRobotCommand(object):
         return self.getState()
 
     def system(self, system):
-        """Move arm to the start of the program.
+        """Get the RobotCommand instance for a specific system.
 
             Args:
                 system(int): id of the system
 
             Returns:
-                RobotCommand: returns a robot command instance for specified system id
+                RobotCommand: robot command instance for specified system id
 
         """
-        return self.__robot_command_list[system]
+        return self.__robot_command_list[system]
\ No newline at end of file
